from . import powo
from .powo_terms import Name, Characteristic, Geography, Filters

def test_basic_search():
    res = powo.search('Poa Annua')
    assert res.size() == 2
    assert next(res)['fqId'] == 'urn:lsid:ipni.org:names:320035-2'

def test_advanced_name_search():
    query = { Name.genus: 'Poa', Name.species: 'annua', Name.author: 'L.' }
    res = powo.search(query)
    assert res.size() == 1
    assert next(res)['fqId'] == 'urn:lsid:ipni.org:names:320035-2'

def test_advanced_characteristic_search():
    query = { Characteristic.flower: 'yellow', Characteristic.leaf: 'alternate' }
    res = powo.search(query)
    assert res.size() > 0

def test_advanced_geography_search():
    res = powo.search({Geography.distribution: 'Africa'})
    assert res.size() > 0

def test_lookup():
    res = powo.lookup('urn:lsid:ipni.org:names:320035-2')
    assert res['name'] == 'Poa annua'

def test_lookup_with_extra_fields():
    res = powo.lookup('urn:lsid:ipni.org:names:320035-2', include=['distribution', 'descriptions'])
    assert 'distribution' in res
    assert 'descriptions' in res

def test_filters():
    q = { Name.family: 'Poaceae' }
    f = Filters.accepted
    unfiltered = powo.search(q)
    filtered = powo.search(q, filters = f)

    assert filtered.size() < unfiltered.size()



from . import ipni
from .ipni_terms import Name, Author, Publication, Filters

def test_basic_search():
    res = ipni.search('Poa Annua')
    assert res.size() == 12
    assert next(res)['id'] == '320035-2'

def test_advanced_name_search():
    query = { Name.genus: 'Poa', Name.species: 'annua', Name.author: 'L.' }
    res = ipni.search(query)
    assert res.size() == 1
    assert next(res)['id'] == '320035-2'

def test_advanced_author_search():
    res = ipni.search({Author.standard_form: 'L.'})
    assert '12653-1' in [author['id'] for author in res]

def test_advanced_publication_search():
    res = ipni.search({Publication.lc_number: 'QK91.S6'})
    assert '1071-2' in [pub['id'] for pub in res]

def test_lookup_name():
    res = ipni.lookup_name('320035-2')
    assert res['name'] == 'Poa annua'

def test_lookup_publication():
    res = ipni.lookup_publication('1071-2')
    assert res['title'] == 'Species Plantarum'

def test_lookup_author():
    res = ipni.lookup_author('12653-1')
    assert res['standardForm'] == 'L.'

def test_filter_by_family():
    q = { Name.family: 'Poaceae' }
    f = Filters.familial
    filtered = ipni.search(query = q, filters = f)

    assert filtered.size() == 1

def test_filter_by_infrafamily():
    q = { Name.family: 'Poaceae' }
    f = Filters.infrafamilial
    unfiltered = ipni.search(query = q)
    filtered = ipni.search(query = q, filters = f)

    assert filtered.size() < unfiltered.size()

def test_filter_by_generic():
    q = { Name.family: 'Poaceae' }
    f = Filters.generic
    unfiltered = ipni.search(query = q)
    filtered = ipni.search(query = q, filters = f)

    assert filtered.size() < unfiltered.size()

def test_filter_by_infrageneric():
    q = { Name.family: 'Poaceae' }
    f = Filters.infrageneric
    unfiltered = ipni.search(query = q)
    filtered = ipni.search(query = q, filters = f)

    assert filtered.size() < unfiltered.size()

def test_filter_by_specific():
    q = { Name.family: 'Poaceae' }
    f = Filters.specific
    unfiltered = ipni.search(query = q)
    filtered = ipni.search(query = q, filters = f)

    assert filtered.size() < unfiltered.size()

def test_filter_by_infraspecific():
    q = { Name.family: 'Poaceae' }
    f = Filters.infraspecific
    unfiltered = ipni.search(query = q)
    filtered = ipni.search(query = q, filters = f)

    assert filtered.size() < unfiltered.size()



from enum import Enum

class Name(Enum):
    full_name = "name"
    common_name = "common name"
    kingdom = "kingdom"
    family = "family"
    genus = "genus"
    species = "species"
    author = "author"

class Characteristic(Enum):
    summary = "summary"
    appearance = "appearance"
    characteristic = "characteristic"
    flower = "flower"
    fruit = "fruit"
    leaf = "leaf"
    inflorescence = "inflorescence"
    seed = "seed"
    cloning = "cloning"
    use = "use"

class Geography(Enum):
    distribution = "location"

class Filters(Enum):
    accepted = "accepted_names"
    has_images = "has_images"
    families = "families_f"
    genera = "genus_f"
    species = "species_f"
    infraspecies = "infraspecific_f"



from .core import Api, IPNI_URL, SearchResult

API = Api(IPNI_URL)

def search(query, filters = None):
    return SearchResult(API, query, filters=filters)

def lookup_name(id):
    return API.get('n/' + id).json()

def lookup_author(id):
    return API.get('a/' + id).json()

def lookup_publication(id):
    return API.get('p/' + id).json()



from enum import Enum

class Name(Enum):
    added = "added"
    author = "name author"
    basionym = "basionym"
    basionym_author = "basionym author"
    bibliographic_reference = "bibliographic reference"
    citation_type = "citation type"
    collection_number = "collection number"
    collectors = "collector team"
    distribution = "distribution"
    family = "family"
    full_name = "full name"
    genus = "genus"
    in_powo = "in powo"
    infrafamily = "infrafamily"
    infragenus = "infragenus"
    infraspecies = "infraspecies"
    modified = "modified"
    name_status = "name status"
    published = "published"
    published_in = "published in"
    publishing_author = "publishing author"
    rank = "rank"
    scientific_name = "scientific name"
    species = "species"
    species_author = "species author"
    version = "version"

class Author(Enum):
    forename = "author forename"
    full_name = "author name"
    standard_form = "author std"
    surname = "author surname"

class Publication(Enum):
    standard_form = "publication std"
    bph_number = "bph number"
    date = "date"
    isbn = "isbn"
    issn = "issn"
    lc_number = "lc number"
    preceded_by = "preceded by"
    superceded_by = "superceded by"
    title = "publication title"
    tl2_author = "tl2 author"
    tl2_number = "tl2 number"

class Filters(Enum):
    familial = "f_familial"
    infrafamilial = "f_infrafamilial"
    generic = "f_generic"
    infrageneric = "f_infrageneric"
    specific = "f_specific"
    infraspecific = "f_infraspecific"



import requests
import time
import urllib

IPNI_URL = 'http://beta.ipni.org/api/1'
POWO_URL = 'http://www.plantsoftheworldonline.org/api/2'
KPL_URL = 'http://kewplantlist.org/api/v1'

class Api:
    def __init__(self, url):
        self._base_url = url

    def _url(self, method, params):
        return '{base}/{method}?{opt}'.format(
                base=self._base_url,
                method=method,
                opt=urllib.parse.urlencode(params))

    def get(self, method, params = {}):
        return requests.get(self._url(method, params))

class SearchResult:
    def __init__(self, api, query, filters = None):
        self._query = query
        self._filters = filters
        self._api = api
        self._cursor = "*"
        self._run_query()

    def _build_params(self):
        params = {'perPage': 500, 'cursor': self._cursor}
        if self._query:
            params['q'] = self._format_query()
        if self._filters:
            params['f'] = self._format_filters()
        return params

    def _format_query(self):
        if isinstance(self._query, dict):
            terms = [k.value + ':' + v for k, v in self._query.items()]
            return ",".join(terms)
        else:
            return self._query

    def _format_filters(self):
        if isinstance(self._filters, list):
            terms = [f.value for f in self._filters]
            return ",".join(terms)
        else:
            return self._filters.value

    def _run_query(self):
        params = self._build_params()
        response = self._api.get('search', params)
        # wait a proportion of server response time of previous call
        # before making subsequent calls
        self._wait_time = response.elapsed.total_seconds() / 2.0
        self._response = response.json()
        if 'results' in self._response:
            self._results = iter(self._response['results'])
        if 'cursor' in self._response:
            self._cursor = self._response['cursor']

    def __iter__(self):
        return self

    def __next__(self):
        try:
            return next(self._results)
        except StopIteration:
            time.sleep(self._wait_time)
            self._run_query()
            return next(self._results)

    def size(self):
        if 'totalResults' in self._response:
            return self._response['totalResults']
        else:
            return 0



from .core import Api, POWO_URL, SearchResult

API = Api(POWO_URL)

def search(query, filters = None):
    return SearchResult(API, query, filters=filters)

def lookup(id, include = None):
    params = {'fields': ','.join(include)} if include else {}
    return API.get('taxon/' + id, params).json()



from .core import Api, KPL_URL, SearchResult

API = Api(KPL_URL)

def search(query = None):
    return SearchResult(API, query=query)

def lookup(id, include = None):
    return API.get('taxon/' + id).json()
